# Generated by roxygen2: do not edit by hand

S3method("$",dm)
S3method("$",dm_zoomed)
S3method("$<-",dm)
S3method("[",dm)
S3method("[",dm_zoomed)
S3method("[<-",dm)
S3method("[[",dm)
S3method("[[",dm_zoomed)
S3method("[[<-",dm)
S3method("length<-",dm)
S3method("names<-",dm)
S3method(anti_join,dm)
S3method(anti_join,dm_keyed_tbl)
S3method(anti_join,dm_zoomed)
S3method(arrange,dm)
S3method(arrange,dm_keyed_tbl)
S3method(arrange,dm_zoomed)
S3method(as.list,dm)
S3method(as.list,dm_zoomed)
S3method(as_dm,DBIConnection)
S3method(as_dm,default)
S3method(as_dm,src)
S3method(collect,dm)
S3method(collect,dm_zoomed)
S3method(compute,dm)
S3method(compute,dm_zoomed)
S3method(copy_to,dm)
S3method(count,dm)
S3method(count,dm_keyed_tbl)
S3method(count,dm_zoomed)
S3method(cross_join,dm)
S3method(cross_join,dm_keyed_tbl)
S3method(cross_join,dm_zoomed)
S3method(db_schema_create,"Microsoft SQL Server")
S3method(db_schema_create,PqConnection)
S3method(db_schema_create,SQLiteConnection)
S3method(db_schema_create,src_dbi)
S3method(db_schema_drop,"Microsoft SQL Server")
S3method(db_schema_drop,PqConnection)
S3method(db_schema_drop,SQLiteConnection)
S3method(db_schema_drop,src_dbi)
S3method(db_schema_exists,"Microsoft SQL Server")
S3method(db_schema_exists,PqConnection)
S3method(db_schema_exists,SQLiteConnection)
S3method(db_schema_exists,src_dbi)
S3method(db_schema_list,"Microsoft SQL Server")
S3method(db_schema_list,Pool)
S3method(db_schema_list,PqConnection)
S3method(db_schema_list,SQLiteConnection)
S3method(db_schema_list,src_dbi)
S3method(dim,dm_zoomed)
S3method(dimnames,dm_zoomed)
S3method(distinct,dm)
S3method(distinct,dm_keyed_tbl)
S3method(distinct,dm_zoomed)
S3method(filter,dm)
S3method(filter,dm_keyed_tbl)
S3method(filter,dm_zoomed)
S3method(filter_out,dm)
S3method(filter_out,dm_keyed_tbl)
S3method(filter_out,dm_zoomed)
S3method(format,dm)
S3method(format,dm_cg_code_block)
S3method(format,dm_keys)
S3method(format,dm_zoomed)
S3method(format,dm_zoomed_df)
S3method(full_join,dm)
S3method(full_join,dm_keyed_tbl)
S3method(full_join,dm_zoomed)
S3method(glimpse,dm)
S3method(glimpse,dm_zoomed)
S3method(group_by,dm)
S3method(group_by,dm_keyed_tbl)
S3method(group_by,dm_zoomed)
S3method(group_data,dm)
S3method(group_data,dm_zoomed)
S3method(group_indices,dm)
S3method(group_indices,dm_zoomed)
S3method(group_keys,dm)
S3method(group_keys,dm_zoomed)
S3method(group_vars,dm)
S3method(group_vars,dm_zoomed)
S3method(groups,dm)
S3method(groups,dm_zoomed)
S3method(head,dm_zoomed)
S3method(inner_join,dm)
S3method(inner_join,dm_keyed_tbl)
S3method(inner_join,dm_zoomed)
S3method(json_nest,"tbl_Microsoft SQL Server")
S3method(json_nest,data.frame)
S3method(json_nest,tbl_lazy)
S3method(json_nest_aggregate,default)
S3method(json_nest_join,data.frame)
S3method(json_pack,"tbl_Microsoft SQL Server")
S3method(json_pack,data.frame)
S3method(json_pack,tbl_lazy)
S3method(json_pack_join,data.frame)
S3method(json_unnest,data.frame)
S3method(json_unpack,data.frame)
S3method(left_join,dm)
S3method(left_join,dm_keyed_tbl)
S3method(left_join,dm_zoomed)
S3method(length,dm)
S3method(length,dm_zoomed)
S3method(mutate,dm)
S3method(mutate,dm_keyed_tbl)
S3method(mutate,dm_zoomed)
S3method(names,dm)
S3method(names,dm_zoomed)
S3method(nest_join,dm)
S3method(nest_join,dm_zoomed)
S3method(pack_join,data.frame)
S3method(pack_join,dm)
S3method(pack_join,dm_zoomed)
S3method(print,dm)
S3method(print,dm_cg_code_block)
S3method(print,dm_examine_cardinalities)
S3method(print,dm_examine_constraints)
S3method(print,dm_zoomed)
S3method(pull,dm)
S3method(pull,dm_zoomed)
S3method(pull_tbl,dm)
S3method(pull_tbl,dm_zoomed)
S3method(reframe,dm)
S3method(reframe,dm_keyed_tbl)
S3method(reframe,dm_zoomed)
S3method(relocate,dm)
S3method(relocate,dm_keyed_tbl)
S3method(relocate,dm_zoomed)
S3method(rename,dm)
S3method(rename,dm_keyed_tbl)
S3method(rename,dm_zoomed)
S3method(right_join,dm)
S3method(right_join,dm_keyed_tbl)
S3method(right_join,dm_zoomed)
S3method(rows_truncate,data.frame)
S3method(rows_truncate,tbl_sql)
S3method(select,dm)
S3method(select,dm_keyed_tbl)
S3method(select,dm_zoomed)
S3method(semi_join,dm)
S3method(semi_join,dm_keyed_tbl)
S3method(semi_join,dm_zoomed)
S3method(separate,dm)
S3method(separate,dm_keyed_tbl)
S3method(separate,dm_zoomed)
S3method(slice,dm)
S3method(slice,dm_keyed_tbl)
S3method(slice,dm_zoomed)
S3method(sql_json_nest,PqConnection)
S3method(sql_json_pack,PqConnection)
S3method(sql_rows_truncate,tbl_SQLiteConnection)
S3method(sql_rows_truncate,tbl_duckdb_connection)
S3method(sql_rows_truncate,tbl_sql)
S3method(src_tbls,dm)
S3method(str,dm)
S3method(str,dm_zoomed)
S3method(summarise,dm)
S3method(summarise,dm_keyed_tbl)
S3method(summarise,dm_zoomed)
S3method(tail,dm_zoomed)
S3method(tally,dm)
S3method(tally,dm_keyed_tbl)
S3method(tally,dm_zoomed)
S3method(tbl,dm)
S3method(tbl_sum,dm_keyed_tbl)
S3method(tbl_sum,dm_zoomed_df)
S3method(tbl_vars,dm)
S3method(tbl_vars,dm_zoomed)
S3method(transmute,dm)
S3method(transmute,dm_keyed_tbl)
S3method(transmute,dm_zoomed)
S3method(ungroup,dm)
S3method(ungroup,dm_keyed_tbl)
S3method(ungroup,dm_zoomed)
S3method(unite,dm)
S3method(unite,dm_keyed_tbl)
S3method(unite,dm_zoomed)
S3method(vec_cast,dm_keys.dm_keys)
S3method(vec_proxy_compare,dm_keys)
S3method(vec_ptype2,dm_keys.dm_keys)
S3method(vec_ptype_abbr,dm_keys)
export("%>%")
export(anti_join)
export(arrange)
export(as_dm)
export(cdm_add_fk)
export(cdm_add_pk)
export(cdm_add_tbl)
export(cdm_apply_filters)
export(cdm_apply_filters_to_tbl)
export(cdm_check_constraints)
export(cdm_copy_to)
export(cdm_disambiguate_cols)
export(cdm_draw)
export(cdm_enum_fk_candidates)
export(cdm_enum_pk_candidates)
export(cdm_filter)
export(cdm_flatten_to_tbl)
export(cdm_get_all_fks)
export(cdm_get_all_pks)
export(cdm_get_available_colors)
export(cdm_get_colors)
export(cdm_get_con)
export(cdm_get_filter)
export(cdm_get_fk)
export(cdm_get_pk)
export(cdm_get_referencing_tables)
export(cdm_get_src)
export(cdm_get_tables)
export(cdm_has_fk)
export(cdm_has_pk)
export(cdm_insert_zoomed_tbl)
export(cdm_is_referenced)
export(cdm_join_to_tbl)
export(cdm_learn_from_db)
export(cdm_nrow)
export(cdm_nycflights13)
export(cdm_paste)
export(cdm_rename)
export(cdm_rename_tbl)
export(cdm_rm_fk)
export(cdm_rm_pk)
export(cdm_rm_tbl)
export(cdm_select)
export(cdm_select_tbl)
export(cdm_set_colors)
export(cdm_squash_to_tbl)
export(cdm_update_zoomed_tbl)
export(cdm_zoom_out)
export(cdm_zoom_to_tbl)
export(check_cardinality)
export(check_cardinality_0_1)
export(check_cardinality_0_n)
export(check_cardinality_1_1)
export(check_cardinality_1_n)
export(check_if_subset)
export(check_key)
export(check_set_equality)
export(check_subset)
export(collect)
export(compute)
export(copy_dm_to)
export(copy_to)
export(db_schema_create)
export(db_schema_drop)
export(db_schema_exists)
export(db_schema_list)
export(decompose_table)
export(dm)
export(dm_add_fk)
export(dm_add_pk)
export(dm_add_tbl)
export(dm_add_uk)
export(dm_apply_filters)
export(dm_apply_filters_to_tbl)
export(dm_bind)
export(dm_ddl_post)
export(dm_ddl_pre)
export(dm_deconstruct)
export(dm_disambiguate_cols)
export(dm_discard_zoomed)
export(dm_dml_load)
export(dm_draw)
export(dm_enum_fk_candidates)
export(dm_enum_pk_candidates)
export(dm_examine_cardinalities)
export(dm_examine_constraints)
export(dm_filter)
export(dm_financial)
export(dm_financial_sqlite)
export(dm_flatten)
export(dm_flatten_to_tbl)
export(dm_from_con)
export(dm_from_src)
export(dm_get_all_fks)
export(dm_get_all_pks)
export(dm_get_all_uks)
export(dm_get_available_colors)
export(dm_get_colors)
export(dm_get_con)
export(dm_get_filters)
export(dm_get_fk)
export(dm_get_pk)
export(dm_get_referencing_tables)
export(dm_get_src)
export(dm_get_table_description)
export(dm_get_tables)
export(dm_gui)
export(dm_has_fk)
export(dm_has_pk)
export(dm_insert_zoomed)
export(dm_is_referenced)
export(dm_join_to_tbl)
export(dm_mutate_tbl)
export(dm_nest_tbl)
export(dm_nrow)
export(dm_nycflights13)
export(dm_pack_tbl)
export(dm_paste)
export(dm_pixarfilms)
export(dm_ptype)
export(dm_rename)
export(dm_rename_tbl)
export(dm_reset_table_description)
export(dm_rm_fk)
export(dm_rm_pk)
export(dm_rm_tbl)
export(dm_rm_uk)
export(dm_rows_append)
export(dm_rows_delete)
export(dm_rows_insert)
export(dm_rows_patch)
export(dm_rows_truncate)
export(dm_rows_update)
export(dm_rows_upsert)
export(dm_select)
export(dm_select_tbl)
export(dm_set_colors)
export(dm_set_table_description)
export(dm_sql)
export(dm_squash_to_tbl)
export(dm_unnest_tbl)
export(dm_unpack_tbl)
export(dm_unwrap_tbl)
export(dm_update_zoomed)
export(dm_validate)
export(dm_wrap_tbl)
export(dm_zoom_to)
export(enum_fk_candidates)
export(enum_pk_candidates)
export(examine_cardinality)
export(filter)
export(full_join)
export(glimpse)
export(group_by)
export(inner_join)
export(is_dm)
export(json_nest)
export(json_nest_join)
export(json_pack)
export(json_pack_join)
export(json_unnest)
export(json_unpack)
export(left_join)
export(mutate)
export(new_dm)
export(pack_join)
export(pull_tbl)
export(rename)
export(reunite_parent_child)
export(reunite_parent_child_from_list)
export(right_join)
export(rows_append)
export(rows_delete)
export(rows_insert)
export(rows_patch)
export(rows_truncate)
export(rows_update)
export(rows_upsert)
export(select)
export(semi_join)
export(separate)
export(sql_rows_truncate)
export(sql_schema_create)
export(sql_schema_drop)
export(sql_schema_exists)
export(sql_schema_list)
export(src_tbls)
export(summarise)
export(summarize)
export(tbl)
export(tibble)
export(transmute)
export(ungroup)
export(unite)
export(validate_dm)
import(cli)
import(dplyr, except = c(data_frame))
import(glue)
import(lifecycle)
import(purrr, except = c(list_along, rep_along, invoke, modify, as_function, flatten_dbl, flatten_lgl, flatten_int, flatten_raw, flatten_chr, splice, flatten, prepend, `%@%`, `%||%`))
import(rlang)
import(tibble, except = c(data_frame))
import(tidyr, except = c(extract))
import(vctrs)
importFrom(dplyr,"%>%")
importFrom(dplyr,anti_join)
importFrom(dplyr,arrange)
importFrom(dplyr,collect)
importFrom(dplyr,compute)
importFrom(dplyr,copy_to)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,rename)
importFrom(dplyr,right_join)
importFrom(dplyr,rows_append)
importFrom(dplyr,rows_delete)
importFrom(dplyr,rows_insert)
importFrom(dplyr,rows_patch)
importFrom(dplyr,rows_update)
importFrom(dplyr,rows_upsert)
importFrom(dplyr,select)
importFrom(dplyr,semi_join)
importFrom(dplyr,src_tbls)
importFrom(dplyr,summarise)
importFrom(dplyr,summarize)
importFrom(dplyr,tbl)
importFrom(dplyr,transmute)
importFrom(dplyr,ungroup)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colors)
importFrom(grDevices,rgb)
importFrom(methods,is)
importFrom(tibble,glimpse)
importFrom(tibble,tibble)
importFrom(tidyr,separate)
importFrom(tidyr,unite)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,str)
importFrom(utils,tail)
