

How to install Accelar/450 Management Software
----------------------------------------------

This file contains instructions to install the Device 
Management(DM) and VLAN Management(VM) software for AIX, 
HPUX and Solaris UNIX platforms.

The standard installation procedure involves copying the
compressed tar file containing the DM/VM software from 
the cdrom onto the user's hard-disk.  The file is then
uncompressed and untared.  Lastly, the user must manually 
add the "DMPATH" environment variable to their shell file.
(See the section "Standard Installation Procedure" below).

Optionally, if the user is running HP OpenView(without
Optivity) and would like to be able to launch the DM/VM 
software directly from the HPOV windows, the user can 
execute one additional script to install the DM/VM software
into HPOV. (See the section "Installing DM/VM into HPOV" 
below).



Standard Installation Procedure 
-------------------------------

Prerequisites

- AIX workstation with AIX 4.1 or higher
- HP workstation with HPUX 10.20 or higher
- SPARC workstaion with Solaris 2.4 or higher 
- 32 MB DRAM
- 3MB space on hard disk


Step #1 :

Copy the compressed tar file containing the DM/VM software 
to the host's hard-disk.  The instructions that follow assumes 
the user has mounted the cdrom drive onto the "/cdrom" directory.
If you do not know how to mount the cdrom drive, refer to your 
system's operation system user guide. 

Execute the following commands : 

a. Create a directory on the host's hard-disk where you
   want the DM/VM software to reside.  The location
   can be anywhere the user chooses.  Typically, 3rd party
   software is installed under the "/opt" directory.
   Depending on how your system is configured, you may need
   to be the root user to have write permission to the "/opt"
   directory.  If you do not have root access, you can install
   the software under "/usr/local/Bay".

   % mkdir /opt/Bay

b. Goto the directory on the cdrom where the DM/VM software is 
   located:

   % cd /cdrom/dm-unix

c. Execute the "install_dmvm" script by providing the version 
   of the software you want to install and target directory where 
   you have decided to install the DM/VM software: 

   % install_dmvm dm_v200 /opt/Bay


   If you encounter problems executing this script, refer
   to the "Trouble Shooting" section at the end of this file.


Step #2 :

Add the "DMPATH" environment variable to your shell file.  
If you are using c-shell, add a line like this to your .cshrc 
file:

setenv DMPATH /opt/Bay/dm_v[version_number]

where [version_number] is the version of the DM/VM software you 
just installed.  For instance, if you are installing version 
1.3.0 of the DM/VM software, the line you need to add would look 
like this:

setenv DMPATH /opt/Bay/dm_v200



Installing DM/VM into HPOV (Optional)
-------------------------------------

Prerequisites

- HP OpenView must be installed.
- Optivity should not be installed.  If you're running Optivity
  you should follow the Optivity installation procedures to
  upgrade your Optivity software with the newer Management
  Software. (Using the instructions that follow will cause Optivity
  to have uninstall problems).
- Device Management software must be installed.
- You must be logged on as root.


To integrate Device Management software:

a. Go to the directory where the Device Management software is 
   installed.

   % cd /opt/Bay/dm_v200

b. As the root user start Device Manager and point it to an 
   Accelar or 450 device (e.g., "dm a.b.c.d", where a.b.c.d is the 
   IP address of the Accelar/450 device or the name of the device 
   if your network is running some name service).

   % dm 132.32.1.5

c. From Device Manager, click on Help from menu bar and select 
   the "Run HP OpenView Install on Console" option.

   This will copy the Accelar/450 specific icons and MIB files to the 
   standard HP Openview software directories.  When this is completed,
   a list of commands will be shown on the console.  Manually execute 
   the commands as root to complete the installation process.

   Here's a brief description of each command:

   ------------------------------------------------------------
   Command 		| 	What it does ...	
   ------------------------------------------------------------
   1. ovw -fields	|	Update the fields database.
   ------------------------------------------------------------
   2. ovstop netmon	|	Stop network monitor to allow the
			|	database to update.
   ------------------------------------------------------------
   3. ovtopofix		| 	Update topology database.	
   ------------------------------------------------------------
   4. ovstart netmon	|	Restart network monitor.
   ------------------------------------------------------------
   5. ovw		|	Start HP OpenView.
   ------------------------------------------------------------

d. After HP Overview is started, load the Accelar/450 specific mibs by
   selecting the "Options->Load MIBs" option.  Load the "rapid_city.mib"
   file located in the "/var/opt/OV/share/snmp_mibs/Vendor" directory.



Trouble Shooting
----------------

o Why am I getting the "permission denied" message when executing the
  "install_dmvm" script?

  Make sure the "install_dmvm" file has executible permission.
  You make need to execute this command:

    % chmod a+x install_dmvm
  

o Why is the "install_dmvm" script failing?

  If you encounter problems executing the script, you can work-around
  the problem by doing the install manually.  Execute the following
  commands:  

     % mkdir /opt/Bay
     % cd /cdrom/dm-unix
     % cp dm_v200.tar.Z  /opt/Bay
     % cd /opt/Bay
     % uncompress dm_v200.tar.Z
     % tar -xv dm_v200.tar
     % cd dm_v200
     % standalone_setup

  Finish-up the installation procedure by performing Step#2 described
  in section "Standard Installation Procedure".


o When I start-up the Device Manager(DM) program, I'm getting this 
  message: "DMPATH variable not set". 
 
  Set the DMPATH environment variable in your shell file by performing
  Step #2 described in section "Standard Installation Procedure".



Revision : 09/02/98 [ET] - initial version
Revision : 11/24/98 [ET] - updated for rel1.3.3
Revision : 12/15/98 [ET] - updated for rel2.0.0


