#' @keywords internal
#' @aliases NULL dm-package
"_PACKAGE"

#' @import rlang
#' @import cli
#' @import glue
#' @import lifecycle
#' @import vctrs
#' @rawNamespace import(tibble, except = c(data_frame))
#' @rawNamespace import(dplyr, except = c(data_frame))
#' @rawNamespace import(purrr, except = c(list_along, rep_along, invoke, modify, as_function, flatten_dbl, flatten_lgl, flatten_int, flatten_raw, flatten_chr, splice, flatten, prepend, `%@%`, `%||%`))
#' @rawNamespace import(tidyr, except = c(extract))
## usethis namespace: start
#' @importFrom methods is
#' @importFrom utils head tail str packageVersion
#' @importFrom grDevices col2rgb colors rgb
## usethis namespace: end
NULL
