# Generated by roxyglobals: do not edit by hand

utils::globalVariables(c(
  "parent_table", # <build_copy_queries>
  "parent_key_cols", # <build_copy_queries>
  "autoincrement", # <build_copy_queries>
  "name", # <build_copy_queries>
  "col_def", # <build_copy_queries>
  "pk_col", # <build_copy_queries>
  "unique_def", # <build_copy_queries>
  "child_table", # <build_copy_queries>
  "child_fk_cols", # <build_copy_queries>
  "on_delete", # <build_copy_queries>
  "fk_def", # <build_copy_queries>
  "remote_name", # <build_copy_queries>
  "remote_name_unquoted", # <build_copy_queries>
  "index_name", # <build_copy_queries>
  "columns", # <build_copy_queries>
  "schema_name", # <get_src_tbl_names>
  "table_name", # <get_src_tbl_names>
  "remote_name", # <get_src_tbl_names>
  "local_name", # <get_src_tbl_names>
  "child_uuid", # <fks_df_from_keys_info>
  "parent_uuid", # <fks_df_from_keys_info>
  "child_fk_cols", # <fks_df_from_keys_info>
  "child_data", # <fks_df_from_keys_info>
  "parent_key_cols", # <fks_df_from_keys_info>
  "parent_data", # <fks_df_from_keys_info>
  "parent_table", # <fks_df_from_keys_info>
  "child_table", # <fks_df_from_keys_info>
  "new_name", # <compute_disambiguate_cols_recipe>
  "column", # <compute_disambiguate_cols_recipe>
  "text", # <explain_col_rename>
  "new_table", # <dm_disentangle>
  "fks", # <dm_disentangle>
  "uks", # <dm_disentangle>
  "child_table", # <dm_flatten_impl>
  "parent_table", # <dm_flatten_impl>
  "child_table", # <dm_flatten_join>
  "parent_table", # <dm_flatten_join>
  "child_table", # <dm_flatten_transfer_fks>
  "parent_table", # <dm_flatten_transfer_fks>
  "pks", # <str.dm>
  "fks", # <str.dm>
  "filters", # <str.dm>
  "zoom", # <str.dm_zoomed>
  "pks", # <str.dm_zoomed>
  "fks", # <str.dm_zoomed>
  "filters", # <str.dm_zoomed>
  "ref_uuid", # <tbl_def_impl>
  "ref_column", # <tbl_def_impl>
  "column", # <tbl_def_impl>
  "name", # <dm_ddl_pre>
  "col_def", # <dm_ddl_pre>
  "uk_def", # <dm_ddl_pre>
  "fk_def", # <dm_ddl_pre>
  "pk_def", # <dm_ddl_pre>
  "remote_name", # <dm_ddl_pre>
  "renames", # <prepare_join>
  "kind", # <dm_get_data_model>
  "child_table", # <dm_get_data_model>
  "child_fk_cols", # <dm_get_data_model>
  "parent_table", # <dm_get_data_model>
  "parent_key_cols", # <dm_get_data_model>
  "id", # <dm_get_data_model>
  "uk_col", # <dm_get_data_model>
  "column", # <dm_get_data_model>
  "key", # <dm_get_data_model>
  "key_fk", # <dm_get_data_model>
  "value", # <dm_get_all_column_types>
  "new_display", # <dm_set_colors>
  "display", # <dm_set_colors>
  "new_child_table", # <rename_unique>
  "child_table", # <rename_unique>
  "new_parent_table", # <rename_unique>
  "parent_table", # <rename_unique>
  "new_table", # <rename_unique>
  "new_child_table", # <split_to_list>
  "child_table", # <split_to_list>
  "new_parent_table", # <split_to_list>
  "parent_table", # <split_to_list>
  "new_table", # <split_to_list>
  "child_cols", # <split_to_list>
  "parent_cols", # <split_to_list>
  "on_delete", # <split_to_list>
  "on_delete", # <dm_examine_cardinalities_impl>
  "parent_table", # <dm_examine_cardinalities_impl>
  "child_table", # <dm_examine_cardinalities_impl>
  "x_label", # <dm_examine_cardinalities_impl>
  "parent_key_cols", # <dm_examine_cardinalities_impl>
  "y_label", # <dm_examine_cardinalities_impl>
  "child_fk_cols", # <dm_examine_cardinalities_impl>
  "cardinality", # <bullets_cardinalities>
  "column", # <dm_examine_constraints>
  "columns", # <dm_examine_constraints>
  "problem", # <print.dm_examine_constraints>
  "kind", # <print.dm_examine_constraints>
  "ref_table", # <print.dm_examine_constraints>
  "into", # <print.dm_examine_constraints>
  "text", # <print.dm_examine_constraints>
  "uk_col", # <check_pk_constraints>
  "kind", # <check_pk_constraints>
  "pk_col", # <check_pk_constraints>
  "candidate", # <check_pk_constraints>
  "why", # <check_pk_constraints>
  "t1", # <check_fk_constraints>
  "child_table", # <check_fk_constraints>
  "child_fk_cols", # <check_fk_constraints>
  "t2", # <check_fk_constraints>
  "parent_table", # <check_fk_constraints>
  "parent_key_cols", # <check_fk_constraints>
  "problem", # <check_fk_constraints>
  "t1_name", # <check_fk_constraints>
  "kind", # <check_fk_constraints>
  "colname", # <check_fk_constraints>
  "t2_name", # <check_fk_constraints>
  "is_key", # <check_fk_constraints>
  "node", # <dm_get_filtered_table>
  "parent", # <dm_get_filtered_table>
  "distance", # <dm_get_filtered_table>
  "filters", # <dm_get_filters_impl>
  "filter_expr", # <dm_get_filters_impl>
  "fks", # <dm_rm_fk_impl>
  "ref_table", # <dm_rm_fk_impl>
  "need_ref", # <dm_rm_fk_impl>
  "column", # <enum_fk_candidates_impl>
  "why", # <enum_fk_candidates_impl>
  "candidate", # <enum_fk_candidates_impl>
  "fks", # <create_graph_from_dm>
  "ref_table", # <create_graph_from_dm>
  "JSON_AGG", # <sql_json_nest.PqConnection>
  "JSON_BUILD_OBJECT", # <sql_json_nest.PqConnection>
  "JSON_BUILD_OBJECT", # <sql_json_pack.PqConnection>
  "key", # <is_unique_key_se>
  "any_na", # <is_unique_key_se>
  ".", # <is_unique_key_se>
  "data", # <is_unique_key_se>
  "schemata", # <dm_learn_from_db>
  "table_catalog", # <dm_learn_from_db>
  "table_schema", # <dm_learn_from_db>
  "table_name", # <dm_learn_from_db>
  "catalog", # <dm_learn_from_db>
  "columns", # <dm_learn_from_db>
  "ordinal_position", # <dm_learn_from_db>
  "constraint_column_usage", # <dm_learn_from_db>
  "key_column_usage", # <dm_learn_from_db>
  "table_constraints", # <dm_learn_from_db>
  "column_name", # <dm_learn_from_db>
  "constraint_type", # <dm_learn_from_db>
  "constraint_catalog", # <dm_learn_from_db>
  "constraint_schema", # <dm_learn_from_db>
  "constraint_name", # <dm_learn_from_db>
  "is_autoincrement", # <dm_learn_from_db>
  "pks", # <dm_learn_from_db>
  "delete_rule", # <dm_learn_from_db>
  "constraint_column_usage.dm_name", # <dm_learn_from_db>
  "constraint_column_usage.column_name", # <dm_learn_from_db>
  "key_column_usage.dm_name", # <dm_learn_from_db>
  "key_column_usage.column_name", # <dm_learn_from_db>
  "ref_table", # <dm_learn_from_db>
  "ref_column", # <dm_learn_from_db>
  "column", # <dm_learn_from_db>
  "fks", # <dm_learn_from_db>
  "extra", # <dm_meta_raw>
  "constraint_catalog", # <dm_meta_raw>
  "constraint_schema", # <dm_meta_raw>
  "table_name", # <dm_meta_raw>
  "constraint_type", # <dm_meta_raw>
  "constraint_name", # <dm_meta_raw>
  "referenced_table_name", # <dm_meta_raw>
  "referenced_table_schema", # <dm_meta_raw>
  "referenced_column_name", # <dm_meta_raw>
  "table_constraints", # <dm_meta_add_keys>
  "constraint_catalog", # <dm_meta_add_keys>
  "constraint_schema", # <dm_meta_add_keys>
  "constraint_name", # <dm_meta_add_keys>
  "table_catalog", # <dm_meta_add_keys>
  "table_schema", # <dm_meta_add_keys>
  "table_name", # <dm_meta_add_keys>
  "tables", # <dm_meta_add_keys>
  "key_column_usage", # <dm_meta_add_keys>
  "ordinal_position", # <dm_meta_add_keys>
  "column_name", # <dm_meta_add_keys>
  "columns", # <dm_meta_add_keys>
  "constraint_column_usage", # <dm_meta_add_keys>
  "schemata", # <dm_meta_simple_add_keys>
  "catalog_name", # <dm_meta_simple_add_keys>
  "schema_name", # <dm_meta_simple_add_keys>
  "tables", # <dm_meta_simple_add_keys>
  "table_catalog", # <dm_meta_simple_add_keys>
  "table_schema", # <dm_meta_simple_add_keys>
  "table_name", # <dm_meta_simple_add_keys>
  "columns", # <dm_meta_simple_add_keys>
  "column_name", # <dm_meta_simple_add_keys>
  "schemata", # <select_dm_meta>
  "catalog_name", # <select_dm_meta>
  "schema_name", # <select_dm_meta>
  "tables", # <select_dm_meta>
  "table_catalog", # <select_dm_meta>
  "table_schema", # <select_dm_meta>
  "table_name", # <select_dm_meta>
  "table_type", # <select_dm_meta>
  "columns", # <select_dm_meta>
  "column_name", # <select_dm_meta>
  "ordinal_position", # <select_dm_meta>
  "column_default", # <select_dm_meta>
  "is_nullable", # <select_dm_meta>
  "is_autoincrement", # <select_dm_meta>
  "table_constraints", # <select_dm_meta>
  "constraint_catalog", # <select_dm_meta>
  "constraint_schema", # <select_dm_meta>
  "constraint_name", # <select_dm_meta>
  "constraint_type", # <select_dm_meta>
  "delete_rule", # <select_dm_meta>
  "key_column_usage", # <select_dm_meta>
  "constraint_column_usage", # <select_dm_meta>
  "DATABASE", # <filter_dm_meta>
  "catalog_name", # <filter_dm_meta>
  "table_catalog", # <filter_dm_meta>
  "schema_name", # <filter_dm_meta>
  "table_schema", # <filter_dm_meta>
  "catalog_name", # <filter_dm_meta_simple>
  "table_catalog", # <filter_dm_meta_simple>
  "schema_name", # <filter_dm_meta_simple>
  "table_schema", # <filter_dm_meta_simple>
  "constraint_catalog", # <mssql_constraint_column_usage>
  "constraint_schema", # <mssql_constraint_column_usage>
  "constraint_name", # <mssql_constraint_column_usage>
  "constraint_type", # <mssql_constraint_column_usage>
  "name", # <mssql_constraint_column_usage>
  "object_id", # <mssql_constraint_column_usage>
  "catalog", # <mssql_constraint_column_usage>
  "table_schema", # <mssql_constraint_column_usage>
  "table_name", # <mssql_constraint_column_usage>
  "column_name", # <mssql_constraint_column_usage>
  "constraint_column_id", # <mssql_constraint_column_usage>
  "tailnum", # <dm_nycflights13>
  "carrier", # <dm_nycflights13>
  "faa", # <dm_nycflights13>
  "origin", # <dm_nycflights13>
  "time_hour", # <dm_nycflights13>
  "dest", # <dm_nycflights13>
  "data", # <dm_paste_select>
  "cols", # <dm_paste_select>
  "film", # <dm_pixarfilms>
  "award_type", # <dm_pixarfilms>
  "genre", # <dm_pixarfilms>
  "category", # <dm_pixarfilms>
  "value", # <dm_pixarfilms>
  "column", # <enum_pk_candidates>
  "columns", # <enum_pk_candidates>
  "column", # <enum_pk_candidates_impl>
  "why", # <enum_pk_candidates_impl>
  "candidate", # <enum_pk_candidates_impl>
  "pks", # <dm_upgrade>
  "ref_table", # <dm_validate>
  "ref_column", # <dm_validate>
  "zoomed", # <dm_discard_zoomed>
  "column", # <cdm_enum_fk_candidates>
  "columns", # <cdm_enum_fk_candidates>
  NULL
))
