#' @export
dplyr::`%>%`

#' @export
dplyr::filter

#' @export
dplyr::mutate

#' @export
dplyr::transmute

#' @export
dplyr::summarise

#' @export
dplyr::summarize

#' @export
dplyr::select

#' @export
dplyr::rename

#' @export
dplyr::group_by

#' @export
dplyr::ungroup

#' @export
dplyr::left_join

#' @export
dplyr::inner_join

#' @export
dplyr::full_join

#' @export
dplyr::semi_join

#' @export
dplyr::anti_join

#' @export
dplyr::right_join

#' @export
dplyr::collect

#' @export
dplyr::compute

#' @export
dplyr::copy_to

#' @export
dplyr::src_tbls

#' @export
dplyr::tbl

#' @export
dplyr::arrange

#' @export
dplyr::rows_insert

#' @export
dplyr::rows_append

#' @export
dplyr::rows_update

#' @export
dplyr::rows_patch

#' @export
dplyr::rows_upsert

#' @export
dplyr::rows_delete

#' @export
tibble::glimpse

#' @export
tibble::tibble

#' @export
tidyr::unite

#' @export
tidyr::separate
